#ifndef GUARD_HCALCALOFLAGLABELS_H
#define GUARD_HCALCALOFLAGLABELS_H

#include "DataFormats/HcalDetId/interface/HcalSubdetector.h"
#include <string>

// Create alias names for all status bits
// These aliases are valid for only the _current release_
// Use the HcalCaloFlagTool (below) for full interpretation
namespace HcalCaloFlagLabels
{
  //subdetector-specific bits defined here (bits 0-15)
  enum HBHEStatusFlag{HBHEHpdHitMultiplicity=0,
                      HBHEPulseShape=1,
		      HSCP_R1R2=2,
		      HSCP_FracLeader=3,
		      HSCP_OuterEnergy=4,
		      HSCP_ExpFit=5,
                      HBHETimingTrustBits=6, // 2-bit counter; not yet in use
                      HBHETimingShapedCutsBits=8 // 3-bit counter
  };

  enum HFTimingTrustFlag{HFTimingTrustBits=6};

  enum HOStatusFlag{HOBit=0};

  enum HFStatusFlag{HFLongShort=0,
		    HFDigiTime=1,
		    HFInTimeWindow=2 // requires hit be within certain time window
  };


  enum ZDCStatusFlag{ZDCBit=0};

  enum CalibrationFlag{CalibrationBit=0};

  // Bit definitions that apply to all subdetectors (bits 16-31)
  enum CommonFlag {TimingSubtractedBit=16, // latency shift correction, recovered
		   TimingAddedBit=17,      // latency shift correction, recovered
		   TimingErrorBit=18,      // latency shift error, unrecovered
		   ADCSaturationBit=19,
                   Fraction2TS=20,
		   // The following bits are all user-defined; reverse-order them so that UserDefinedBit0 will be the last removed
		   UserDefinedBit2 = 29,
		   UserDefinedBit1 = 30,
		   UserDefinedBit0 = 31
}; 
  
}

/** \brief Provides interpretation of flag bits with understanding of 
    CMSSW version dependence.
*/
class HcalCaloFlagTool {
public:
  HcalCaloFlagTool(const std::string& releaseName);
  std::string getFieldName(HcalSubdetector sd, int bit) const;
  int getFieldWidth(HcalSubdetector sd, int bit) const;
  int getFieldStart(HcalSubdetector sd, const std::string& field) const;
  int getFieldWidth(HcalSubdetector sd, const std::string& field) const;
  bool hasField(HcalSubdetector sd, const std::string& field) const;
  bool hasField(HcalSubdetector sd, int bit) const;
private:
  std::string releaseName_;
  bool standardFormat_;
  int major_, minor_, patch_, subpatch_;
};

#endif
